package GeoDB::Display::Dump;

use strict;
use GeoDB::Display;
use GeoDB::Utils;

our @ISA = qw(GeoDB::Display);

our $VERSION = "0.71";

our @GEOOPTIONS = 
  (
   { name => 'short',
     title => 'Display shortened strings:',
     help => 'If specified, limits output to 1 line per field',
     type => 'checkbox'
   },
   { name => 'includeall',
     title => 'Include blank items:',
     type => 'checkbox',
     help => 'If set then all waypoint items will be included, even if the field itself is blank',
     default => 0,
   },
   { name => 'format',
     title => 'Output Format:',
     type => 'menu',
     values => ['text','perl'],
     help => 'If set to "perl" will use perl\'s Data::Dumper to dump the perl object' },
  );

sub display {
     my ($self, $set, $dbh) = @_;
     $set->load() if (!$self->{'options'}{'noload'});

     my $out;
     GEODEBUG(3, "Dumping a set\n");
     if ($self->{'options'}{'format'} eq 'perl') {
	 require Data::Dumper;
	 import Data::Dumper qw(Dumper);
	 return Dumper($set);
     }

     my $short = $self->{'options'}{'short'};
     $set->foreach(
		   sub {
		       my ($cache, $short) = @_;
		       $out .= "\nCACHE: $cache->{data}{ident}\n";
		       $out .= "DATA:\n";
		       foreach my $data (keys(%{$cache->{'data'}})) {
			   $out .= 
			     sprintf("  %-30s: %s\n",  $data,
				     (($short) ? substr($cache->{'data'}{$data}, 0, 45) : $cache->{'data'}{$data}))
			       if ($cache->{'data'}{$data} ||
				   $self->{'options'}{'includeall'});
		       }
		       $out .= "ATTRIBUTES:\n";
		       foreach my $data (keys(%{$cache->{'attributes'}})) {
			   $out .= sprintf("  %-30s: %s\n",  $data,
					   (($short) ? substr($cache->{'attributes'}{$data},0,45) : $cache->{'attributes'}{$data}))
		       }
		   }, $short
		  );
     return $out;
 }


1;

=head1 NAME

Dump - Dump the contents of the search set

=head1 DESCRIPTION

START HELP

  Example 1:
  geoqo -d dump

  Example 2:
  geoqo -d dump:short=1

  Example 3:
  geoqo -d dump:format=perl

The I<Dump> display type dumps the contents of a set of waypoints.
This is primarily a debugging mechanism.

Options:

=over

=item short=1

Makes everything fit on one 80column line per item.  This will
truncate field values to something short enough to do this.

=item format=perl

Uses Data::Dumper to show the output in perl structure form.  For the expert.

=back

END HELP

=cut

