package GeoDB::Export::Density;

use strict;
use GeoDB::Utils;

sub get_density_map {
    my ($self, $set, $fh, $sub) = @_;

    # width/height of the image
    my $binnumx =
      $self->{'options'}{'width'} || $self->{'options'}{'size'};
    my $binnumy =
      $self->{'options'}{'height'} || $self->{'options'}{'size'};

    # radius not diameter
    my $spread = $self->{'options'}{'spread'};

    my ($maxlat, $minlat, $maxlon, $minlon) = (-10000, 10000, -10000, 10000);
    $set->foreach(sub {
		      $maxlat = max($_[0]{'data'}->{'lat'}, $maxlat);
		      $minlat = min($_[0]{'data'}->{'lat'}, $minlat);
		      $maxlon = max($_[0]{'data'}->{'lon'}, $maxlon);
		      $minlon = min($_[0]{'data'}->{'lon'}, $minlon);
		  });

    # let user override certain ones
    $maxlat = $self->{'options'}{'nmax'} || $maxlat;
    $minlat = $self->{'options'}{'nmin'} || $minlat;
    $maxlon = $self->{'options'}{'wmax'} || $maxlon;
    $minlon = $self->{'options'}{'wmin'} || $minlon;

    ($maxlat, $maxlon) = parse_coords($maxlat, $maxlon);
    ($minlat, $minlon) = parse_coords($minlat, $minlon);

    GEODEBUG(4,"boundaries: $maxlat, $minlat, $maxlon, $minlon\n");
    my $latdiff = $maxlat - $minlat;
    my $londiff = $maxlon - $minlon;


    GEODEBUG(5,"filling circle\n");
    my $textcir;
    my @spread;
    for (my $i = 0; $i <= $spread*2; $i++) {
	for (my $j = 0; $j <= $spread*2; $j++) {
	    my $xd = $spread-$i;
	    my $yd = $spread-$j;
	    $spread[$i][$j] = (sqrt($xd*$xd + $yd*$yd) <= $spread) ? 1 : 0;
	    if ($spread[$i][$j]) {
		$textcir .= "X";
	    } else {
		$textcir .= ".";
	    }
	}
	$textcir .= "\n";
    }
    GEODEBUG(7,"circle:\n$textcir");

    GEODEBUG(5, "putting caches into bin buckets\n");

    my ($x, $y, @results);

    $set->foreach(
		  sub {
		      $x = int(($_[0]{'data'}{'lat'} - $minlat) * $binnumx / $latdiff);
		      $y = int(($_[0]{'data'}{'lon'} - $minlon) * $binnumy / $londiff);
		      $results[$x][$y]++;
		  });

    my @spreadresults;
    my $maxcount = 0;
    for (my $i = 0; $i <= $binnumx; $i++) {
	for (my $j = 0; $j <= $binnumy; $j++) {
	    for (my $x = $i-$spread; $x <= $i+$spread; $x++) {
		for (my $y = $j-$spread; $y <= $j+$spread; $y++) {
		    if ($spread[$x - $i + $spread][$y - $j + $spread]) {
			if ($x >= 0 && $x <= $#results
			    && $y >= 0 && $y <= $#results) {
			    $spreadresults[$i][$j] += $results[$x][$y];
			}
		    }
		}
	    }
	    $maxcount = max($maxcount,$spreadresults[$i][$j])
	}
    }

#    use Image::Imlib2;
#     my $image = Image::Imlib2->new($binnumx+1, $binnumy+1);
#     $image->set_color(255,255,255,255);
#     $image->draw_rectangle(0, 0, $binnumx+1, $binnumy+1);
#     $image->has_alpha(1);

    GEODEBUG(5, "Finally writting out results\n");
    my $dump = $self->{'options'}{'dump'};
    my $alt = $self->{'options'}{'altitude'};
    my $extrude = $self->{'options'}{'extrude'};
    my $opaque = $self->{'options'}{'opaque'};
    $opaque = int(255*$opaque/100);
    my $linewidth = $self->{'options'}{'linewidth'};
    my $dozeros = $self->{'options'}{'doempty'};

    my $count;
    print $fh "<Folder><name>Cache Density</name>\n";
    for (my $i = 0; $i <= $binnumx; $i++) {
	for (my $j = 0; $j <= $binnumy; $j++) {
	    $count++;
	    if ($spreadresults[$i][$j] > 0 || $dozeros) {
		my $color = sprintf("%02x%02x%02x%02x", 
				    reverse(hsv_to_rgb((($maxcount == 0) ? 0 : .8*$spreadresults[$i][$j] / $maxcount)), $opaque));
		print $fh "
   <Placemark id=\"GeoCD$count\">
    <name>GeoCD$count</name>
    <Style>
      <PolyStyle>
        <color>$color</color>
      </PolyStyle>
      <LineStyle>
        <width>$linewidth</width>
        <color>$color</color>
      </LineStyle>
    </Style>
    <Polygon id=\"poly$count\">
      <extrude>$extrude</extrude>
      <tessellate>1</tessellate>
      <altitudeMode>absolute</altitudeMode>
      <outerBoundaryIs>
        <LinearRing>
          <extrude>0</extrude>
          <tessellate>0</tessellate>
          <altitudeMode>clampToGround</altitudeMode>
          <coordinates>
";
		print $fh
		  ($minlon+$j/$binnumy*$londiff , "," ,
		   $minlat+$i/$binnumx*$latdiff , ",$alt " , 
		   $minlon+($j+1)/$binnumy*$londiff , "," ,
		   $minlat+$i/$binnumx*$latdiff , ",$alt " , 
		   $minlon+($j+1)/$binnumy*$londiff , "," ,
		   $minlat+($i+1)/$binnumx*$latdiff , ",$alt " , 
		   $minlon+$j/$binnumy*$londiff , "," ,
		   $minlat+($i+1)/$binnumx*$latdiff , ",$alt " , 
		   $minlon+$j/$binnumy*$londiff , "," ,
		   $minlat+$i/$binnumx*$latdiff , ",$alt ", 
		   "\n");
		print $fh "          </coordinates>
        </LinearRing>
      </outerBoundaryIs>
    </Polygon>
  </Placemark>\n";
	    }
	    print int(9*$spreadresults[$i][$j]/$maxcount) if ($dump);
# 	    $image->set_color(127,hsv_to_rgb((($maxcount == 0) ? 0 : .8*$spreadresults[$i][$j] / $maxcount)));
# 	    $image->draw_point($i,$j);
# 	    $image->image_orientate(3);
	}
	print "\n" if ($dump);
    }
    print $fh "</Folder>\n";
#    $image->save("tmp.jpg");
}

sub hsv_to_rgb {
    my $h = shift;
    $h = $h * 6;
    my $s = 1;
    my $v = 1;
    my $i = int($h);
    my $f = $h - $i;
    if ($i % 2 == 0) {
	$f = 1 - $f;
    }
    my $m = $v * (1 - $s);
    my $n = $v * (1 - $s * $f);
    return ($v * 255, $n * 255, $m * 255) if ($i == 6 || $i == 0);
    return ($n * 255, $v * 255, $m * 255) if ($i == 1);
    return ($m * 255, $v * 255, $n * 255) if ($i == 2);
    return ($m * 255, $n * 255, $v * 255) if ($i == 3);
    return ($n * 255, $m * 255, $v * 255) if ($i == 4);
    return ($v * 255, $m * 255, $n * 255) if ($i == 5);
}

1;
