package GeoDB::Import::Wigle;

require GeoDB::Import;
use GeoDB::Utils;
use GeoDB::Set;
use GeoDB::Waypoint::Wigle;
use XML::Simple;
use strict;

our $VERSION = "0.71";

our @ISA = qw(GeoDB::Import);

our @GEOOPTIONS = 
({ name => 'setname',
   help => 'Uses setname as the name of geoqo SET to create based on the data.'
 });

sub import_set {
    my ($self, $file, $dbh, %opts) = @_;
    GEODEBUG(1,"reading file: $file\n");

    open(WIG, "<$file");
    my $bogusline = <WIG>;  # dump starting headers

    my $set = new GeoDB::Set;
    # get the name
    my $name = $self->{'options'}{'setname'} || ("import:$file");
    GEODEBUG(2,"processing file: $file into set $name\n");

    # read in each waypoint (cache)
    my $count = 0;

    my ($waypoint);
    while (<WIG>) {
	chomp();
	my @a = split(/~/, $_);
	GEODEBUG(3, "importing record #" . 
		 $count++ . ": $a[3]\n");
	my $id = $a[3];
	$id =~ s/://g;
	$id = "WF$id";

	$waypoint =
	  new GeoDB::Waypoint::Wigle
	    ({
	     data => 
	     {
	      ident =>                         "$id",
	      type  =>                         'Wigle',
	      'lat' => 	     		       "$a[0]",
	      'lon' => 	     		       "$a[1]",
	      'wigle_ssid' => 	     	       "$a[2]",
	      'wigle_netid' => 	     	       "$a[3]",
	      'wigle_discoverer' =>            "$a[4]",
	      'wigle_channel' =>      	       "$a[5]",
	      'wigle_type' =>                  "$a[6]",
	      'wigle_freenet' =>      	       "$a[7]",
	      'wigle_firsttime' =>             "$a[8]",
	      'wigle_wep' => 	               "$a[9]",
	      'wigle_comment' =>      	       "$a[10]",
	      'wigle_qos' =>                   "$a[11]",
	      'wigle_lastupdt' =>      	       "$a[12]",
	      'wigle_paynet' =>      	       "$a[13]",
	      'wigle_userfound' =>             "$a[14]",
	     }});
	$set->add($waypoint);

    }

    $set->set_name($name) if ($name);
    return $set;
}

1;

=cut

=head1 NAME

Wigle - import wigle files

=head1 DESCRIPTION

START HELP

  > geoqo -i myfile.wigle

  > geoqo -i wigle:setname=myname:mywiglefile

The wigle importer imports wigle files taken from the geocaching.com and
waymarking.com site (and probably many other sites too, but only
geocaching.com and waymarking.com WIGLE files have been tested so for)
and imports the data into your geoqo database.

END HELP

=head3 Options:

=over

=item setname=NAME

Uses setname as the name of geoqo SET to create based on the data.
Normally this is the file prepended with "import:".  IE, if you had
downloaded a .wigle file and named it "mywigle.wigle" then when you imported
the wigle file into geoqo you'd get a new set named "import:mywigle.wigle".

=back

=head1 AUTHOR

Wes Hardaker (AKA Yamar) <hardaker ATAT users.sourceforge.net> 

=cut
