package GeoDB::Modify::Gctodiff;

use strict;
use GeoDB::Modify;

our @ISA = qw(GeoDB::Modify);
our $VERSION = "0.71";
our %ratemap = qw(1 1 1.5 2 2 3 2.5 4 3 5 3.5 6 4 7 4.5 8 5 9);

sub modify {
    my ($self, $cache) = @_;
    $cache->load({ noattributes => 1, nologs => 1});
    return if (!exists( $cache->{'data'}{'groundspeak_difficulty'}) ||
                  !exists( $cache->{'data'}{'groundspeak_terrain'}));
    $cache->{'data'}{'newident'} = $cache->{'data'}{'ident'};
    $cache->{'data'}{'newident'} =~ s/../$ratemap{$cache->{'data'}{'groundspeak_difficulty'}}$ratemap{$cache->{'data'}{'groundspeak_terrain'}}/;
}

1;

=head1 NAME

Gctodiff - Modifes the cache identifier to include difficulty/terrain info

=head1 DESCRIPTION

START HELP

The "gctodiff" plugin (short for "GC to difficulty") replaces the
first two letters in the 6-digit cache identifier with a single digit
representation of both the difficulty and terrain ratings.

IE, if you had this cache:

    > geoqo -l
    num GCID     Name
    1   GCQEBQ   Yamar's Unusual #6: The ORB

It's rated a 1.5 in difficulty and a 2 in terrain.  If you ran the
gctodiff modify plugin on this data, it would replace the 'GC' with a
2 and 3:

    > geoqo -m gctodiff: -d 'list:format=%{newident} %{desc}'
    23QEBQ   Yamar's Unusual #6: The ORB

It actually stores the data in the 'newident' field so the old one is
still available.  The export modules, however, use the newident field
when writing out the cache identifiers so this is what your GPS, or
what-have-you, will see when you load it into that.

I (Yamar) do this specifically so when I'm looking at my garmin, for
example, I get an immediate feel for how hard a particular cache is
going to be without having to pull up data about it.  (I also use the
symbol plugin to change the symbol so I can see the type of cache it
is too).  My goal here is to maximize the amount of data I seen on the
screen!

The map from groundspeak/geocaching.com ratings is as follows:

  1    => 1
  1.5  => 2
  2    => 3
  2.5  => 4
  3    => 5
  3.5  => 6
  4    => 7
  4.5  => 8
  5    => 9

END HELP

=cut
