package GeoDB::Modify::Rewrite;

use strict;
use GeoDB::Modify;

our @ISA = qw(GeoDB::Modify);

our $VERSION = "0.71";

sub modify {
    my ($self, $cache) = @_;

    $cache->load({ nologs => 1});
    foreach my $arg (@{$self->{'optionsarray'}}) {
        my ($token, $value) = @$arg;
        $value =~ s/\%{attr:([^\}]+)}/$cache->{'attributes'}{$1}/eg;
        $value =~ s/\%{(\d*)(:|)([^\}]+)}/sprintf("%" . ($1 ? "-$1.$1" : "") . "s", $cache->{'attributes'}{$3} || $cache->{'data'}{$3})/eg;
        $cache->{'data'}{$token} = $value;
    }
}

1;

=head1 NAME

Rewrite - Rewrites fields based on data from the cache

=head1 DESCRIPTION

START HELP

The rewrite modify plugin lets you rewrite fields based on other
fields.  For example, I (Yamar) use the following rewrite when putting
stuff to a gpx file I'm going to load into my TomTom so I get as much
information as possible when clicking on the icon in the tomtom map
display:

  rewrite:groundspeak_name=%{ident}/%{4:subtype}: %{groundspeak_name}

That way the groundspeak_name which might have been "some cache"
becomes "GCABCD/Trad: some cache" to give me more information about
it.  (actually, I use the gctodiff plugin too and use the newident
field instead; see the gctodiff plugin for details).

END HELP

=cut
