package GeoDB::Modify::Solved;

use strict;
use GeoDB::Modify;
use GeoDB::Utils;

our @ISA = qw(GeoDB::Modify);

our $VERSION = "0.71";

our @GEOOPTIONS = 
  (
   { name => 'nattr',
     help => 'The name of the attribute storing the new north coordinate',
     default => 'puzn'},
   { name => 'wattr',
     help => 'The name of the attribute storing the new west coordinate',
     default => 'puzw'}
  );


sub modify {
    my ($self, $cache) = @_;

    $cache->load({nologs => 1});
    my $n = $self->{'options'}{'nattr'};
    my $w = $self->{'options'}{'wattr'};


    if ($cache->{'attributes'}{$n} &&
	$cache->{'attributes'}{$w}) {
	my ($n, $w) = parse_coords($cache->{'attributes'}{$n},
				   $cache->{'attributes'}{$w});
	GEOERR("failed to parse $cache->{data}{ident} solution coord either $self->{'options'}{'nattr'} = $cache->{'attributes'}{$n} or $self->{'options'}{'wattr'} = $cache->{'attributes'}{$w}") if (!$n || !$w);
	$cache->{'data'}{'lat'} = $n;
	$cache->{'data'}{'lon'} = $w;
    }
}

1;

=head1 NAME

Solved - Modifes the lat/lon coordinates to reflect solved puzzle coordinates

=head1 DESCRIPTION

START HELP

Example of solving a puzzle and saving it:

  > geoqo -s ident:GCABCD -m 'setattr:puzn=N38 42.444,puzw=W121 42.444' --save

Example of exporting it with the replaced coordinates:

  > geoqo -s ident:GCABCD -e 'solved:+loc:out.loc'

This plugin replaces the latitude and longitude coordinates of a
waypoint with those in the puzn and puzw attributes.  Use the above
examples as a guide for solving a puzzle and then using the solved:
plugin when exporting your database ready for uploading into a GPS.

END HELP

=cut
