package GeoDB::Modify::Tag;

use strict;
use GeoDB::Modify;
use GeoDB::Utils;

our @ISA = qw(GeoDB::Modify);
our $VERSION = "0.71";

our $VERSION = "0.71";

our @GEOOPTIONS =
  (
   { name => 'tags',
     title => 'List of tags to assign',
     helpdesc => '(separated by / marks)',
   },
   { name => 'deletetags',
     title => 'remove these tags:',
     helpdesc => '(separated by / marks)',
   },
   { name => 'publish',
     type => 'checkbox',
     title => 'Publish tags to tag server:',
   },
   { name => 'publishtags',
     indent => 1,
     type => 'checkbox',
     title => 'Publish only certain tags:',
     helpdesc => '(slash "/" delimited)'
   },
   { name => 'ignoretags',
     indent => 1,
     type => 'checkbox',
     title => 'Don\'t publish these tags:',
     helpdesc => '(slash "/" delimited)',
     default => 'lamppost/found',
   },
   { name => 'fetch',
     type => 'checkbox',
     title => 'Fetch publicized tags from tag server:',
   },
   { name => 'geoqouser',
     title => 'GeoQO Tag Server User Name: ',
   },
   { name => 'geoqopass',
     title => 'GeoQO Tag Server Password: ',
     type => 'hidetext',
   },
  );

sub set_up {
    my ($self, $set) = @_;
    $self->{'taglist'} = [split(/\//,$self->{'options'}{'tags'})];
    $self->{'deletetags'} = [split(/\//,$self->{'options'}{'deletetags'})];
    GEODEBUG(3,"doing tag list: " . join(",",@{$self->{'taglist'}}) . "\n");

    my %tagpublish;
    if ($self->{'options'}{'publishtags'}) {
	my @taglist = split("/",$self->{'options'}{'publishtags'});
	%tagpublish = map { $_, $_ } @taglist;
	$self->{'tagpublish'} = \%tagpublish;
    }

    my %ignoretags;
    if ($self->{'options'}{'ignoretags'}) {
	my @taglist = split("/",$self->{'options'}{'ignoretags'});
	%ignoretags = map { $_, $_ } @taglist;
	$self->{'ignoretags'} = \%ignoretags;
    }

}

sub modify {
    my ($self, $cache) = @_;

    $cache->load({ nologs => 1, noattributes => 1});
    if ($self->{'options'}{'deletetags'}) {
	GEODEBUG(5, "deleting tags: $self->{'options'}{'deletetags'}\n");
	foreach my $tag (@{$self->{'deletetags'}}) {
	    next if (($self->{'options'}{'publishtags'} &&
		      (!exists($self->{'tagpublish'}{$tag}))) ||
		     ($self->{'options'}{'ignoretags'} &&
		      exists($self->{'ignoretags'}{$tag})));
	    $cache->remove_tag($tag);
	    GEODEBUG(6,"removing tag: '$tag' on $cache->{'data'}{'ident'}\n");
	}
    } else {
	foreach my $tag (@{$self->{'taglist'}}) {
	    $cache->set_tag($tag);
	    GEODEBUG(6,"setting tag: '$tag' on $cache->{'data'}{'ident'}\n");
	}
    }
    if ($self->{'options'}{'publish'}) {
	my $publishstr = "$cache->{data}{ident}";
	my $count = 0;
	foreach my $tag (keys(%{$cache->{'tags'}})) {
	    next if ($tag =~ /predicted:/); # XXX: make optonal
	    next if (($self->{'options'}{'publishtags'} &&
		      !exists($self->{'tagpublish'}{$tag})) ||
		     ($self->{'options'}{'ignoretags'} &&
		      exists($self->{'ignoretags'}{$tag})));
	    if ($cache->{'tags'}{$tag}{'needspublished'}) {
		if ($cache->{'tags'}{$tag}{'ismine'}) {
		    $publishstr .= ":$tag";
		} else {
		    $publishstr .= ":-$tag";
		}
	    }
	}
	$self->{'publishstr'} .= "/$publishstr"
	  if ($publishstr ne $cache->{'data'}{'ident'});
    }
    if ($self->{'options'}{'fetch'}) {
	$self->{'fetchstr'} .= "/$cache->{'data'}{'ident'}";
    }
}

sub clean_up {
    my ($self, $set) = @_;
    # XXX: write publish stuff here

    if ($self->{'options'}{'publish'} && $self->{'publishstr'}) {
	my $user = $self->{'options'}{'geoqouser'};
	my $pass = $self->{'options'}{'geoqopass'};
	my $server = $self->{'options'}{'geoqoupserver'} ||
	  'tagsup1.geoqo.org';

	GEOERR("either geoqouser ($user) or geoqopass ($pass) or both wasn't set")
	  if (!$user || !$pass);

	# publish to the geoqo server
	maybe_load('LWP::UserAgent',
		   'The LWP::UserAgent perl module is required for uploading/downloading of tags');
	my $ua = LWP::UserAgent->new(env_proxy => 1);
	my $req = HTTP::Request->new(POST => "http://$server/");
	$req->content_type('application/x-www-form-urlencoded');

	# create the content string
	# user=USER&pass=PASS&set=cache1:tag1:tag2/cache2:tag2:tag3

	$self->{'publishstr'} =~ s/^\///;
	$req->content("user=$user&pass=$pass&set=$self->{'publishstr'}");
	my $res = $ua->request($req);

	GEODEBUG(1,"sent requested tags to the tag server.  Status=" . $res->status_line . "\n");
	GEODEBUG(9,"sent tag string: $self->{'publishstr'}\n");
    }

    if ($self->{'options'}{'fetch'} && $self->{'fetchstr'}) {
	my $user = $self->{'options'}{'geoqouser'};
	my $pass = $self->{'options'}{'geoqopass'};
	my $server = $self->{'options'}{'geoqoupserver'} ||
	  'tagsdown1.geoqo.org';

	GEOERR("either geoqouser ($user) or geoqopass ($pass) or both wasn't set")
	  if (!$user || !$pass);

	# publish to the geoqo server
	maybe_load('LWP::UserAgent',
		   'The LWP::UserAgent perl module is required for uploading/downloading of tags');
	my $ua = LWP::UserAgent->new(env_proxy => 1);
	my $req = HTTP::Request->new(POST => "http://$server/");
	$req->content_type('application/x-www-form-urlencoded');

	# create the content string
	# user=USER&pass=PASS&fetch=cache1/cache2

	$self->{'fetchstr'} =~ s/^\///;
	$req->content("user=$user&pass=$pass&fetch=$self->{'fetchstr'}");
	my $res = $ua->request($req);

	require XML::Simple;
	import XML::Simple qw(XMLin);
	my $tags =
	  XMLin($res->content, ForceArray => [qw(tag tags)]);

	$tags = $tags->{'tags'};

	# update our local DB
	my $dbh = $self->get_master()->{'dbh'};
	GEOERR("can't find local database handle\n") if (!$dbh);

	my $insh = $dbh->prepare("insert into tags(ident,tagname,ismine,needspublished,count,datefetched) values(?,?,0,0,?,date())");
	my $uph = $dbh->prepare("update tags set count = ? where ident = ? and tagname = ?");
	
	# process each received tag
	foreach my $tag (@$tags) {
	    my $cache = $set->find_by_ident($tag->{'ident'});
	    foreach my $tagname (keys(%{$tag->{'tag'}})) {

		# update the local structure as well as the DB
		if (exists($cache->{'tags'}{$tagname})) {
		    $cache->{'tags'}{$tagname}{'count'} = 
		      $tag->{'tag'}{$tagname}{'count'};
		    $uph->execute($tag->{'tag'}{$tagname}{'count'},
				  $tag->{'ident'}, $tagname);
		} else {
		    $cache->{'tags'}{$tagname} = 
		      {
		       count => $tag->{'tag'}{$tagname}{'count'},
		       ismine => 0,
		       needspublished => 0
		      };
		    $insh->execute($tag->{'ident'}, $tagname,
				   $tag->{'tag'}{$tagname}{'count'});
		}
	    }
	}

	GEODEBUG(1,"fetched tags from the tag server.  Status=" . $res->status_line . "\n");
    }
}


1;

=head1 NAME

Tag - Add tags to caches

=head1 DESCRIPTION

START HELP

The "Tag" plugin allows you to add "tag"s to your caches.

Example:

  geoqo -s cache:owner_name==Yamar -m 'tag:tags=awesome/cool'

Another to remove tags:

  geoqo -s cache:owner_name==Yamar -m 'tag:tags=lame,deletetags=1'

=head3 Publishing tags

Your tags may be published to a geoqo tag server, and you may pull
information about how other people have tagged waypoints from the tag
server.  To do this you must first create an account on the tag server:

  http://tagserver.geoqo.org/

Then put your created username and password into your
$HOME/.geoqo/config file like:

  geoqouser YOURUSERNAME
  geoqopass YOURPASSWORD

Then you can publish your tags for a given set using:

  geoqo -s SEARCHTERMS -m tag:publish=1

Additionally you can fetch data about other people's tags using:

  geoqo -s SEARCHTERMS -m tag:fetch=1

When you use the -S option to show a the data for a waypoint you will
see a list of all your tags as well as a list of other peoples tags
that you have fetched.

END HELP

=cut
