package GeoDB::Search::Cache;

use strict;
use GeoDB::Search;
use GeoDB::Utils;
use GeoDB::Waypoint;

our @ISA = qw(GeoDB::Search);

our $VERSION = "0.71";

sub search {
     my ($self, $criteria, $type) = @_;

     GEODEBUG(3, "passing criteria: $criteria\n");
     return
       $self->get_search_tokens( $criteria, $type,
				 get_waypoint_type_cols_and_map('Geocache'),
				 "type == 'Geocache' and");
}


1;

=head1 NAME

Cache - query the waypoint list for geocache attributes

=head2 DESCRIPTION

START HELP

  > geoqo -s cache:name=unusual -d count
  Search/Set Count: 6

Searchs the database for only waypoints that are of type 'Geocache'
and conform to the terms given.  See the documentation on the
waypoint: search type for further details on how to use it because
everything else is the same.

Possible search columns for geocaches include the following, beyond
those listed in the waypoint section:

	available
	archived
	groundspeak_name
	groundspeak_placed_by
	groundspeak_owner_id
	groundspeak_owner_name
	groundspeak_type
	groundspeak_container
	groundspeak_difficulty
	groundspeak_terrain
	groundspeak_country
	groundspeak_state
	groundspeak_short_description_html
	groundspeak_short_description
	groundspeak_long_description_html
	groundspeak_long_description
	groundspeak_encoded_hints

END HELP

=head1 AUTHOR

Wes Hardaker (AKA Yamar) <hardaker ATAT users.sourceforge.net> 

=cut

