package GeoDB::Search::Log;

=cut

=head1 NAME

Log - search waypoint logs

=head1 SYNOPSIS

  > geoqo -s log:logger=Yamar -d count
  Search Result Count: 27

=head1 DESCRIPTION

START HELP

The I<log:> search type lets you search waypoint log entries for
waypoints matching a criteria.  As an example, you can search for all
waypoints in you database found by Yamar (me):

      > geoqo -s 'log:finder=Yamar' -d list
      num GCID     Name
        1 GCJVTR   Here's to Your Health Cache
        ...

Log tokens to search by:

   id
   ident
   date
   type
   logger_id
   logger
   text_encoded
   text

Special tokens to search by (makes sure a log was made by someone of a
particular type):

   finder
   noter
   dnfer

WARNING: this may be amazingly slow on a mysql server.  I think.
Postgresql or sqlite2 (the defualt) database backends are just zippy,
however.

END HELP

=head1 AUTHOR

Wes Hardaker (AKA Yamar) <hardaker ATAT users.sourceforge.net> 

=cut

use GeoDB::Utils;

use strict;

our @ISA = qw(GeoDB::Search);

our $VERSION = "0.71";

our @log_cols = qw(
		   id
		   ident
		   date
		   type
		   logger_id
		   logger
		   text_encoded
		   text

		   finder
		   noter
		   dnfer
);

our %log_map = (
		logger    => 'finder',
		logger_id => 'finder_id',
		finder    => 'finder ARG ? and type = \'Found it\'',
		noter     => 'finder ARG ? and type = \'Write note\'',
		dnfer     => 'finder ARG ? and type = \'Didn\'\'t find it\'',
	       );

sub search {
    my ($self, $criteria, $type) = @_;

    # invert the query if negative to use 'not in' insteaf
    my $not = '';
    $not = " not " if ($criteria =~ s/<(=*)>/=$1/g);

    my ($clause, $args) =
      $self->get_search_tokens( $criteria, $type, \@log_cols, \%log_map);

    $clause = "waypoints.ident $not in (select ident from logs where $clause)";

    GEODEBUG(4,"log query: $clause\n");

    return  ($clause, $args);
}

1;
