package GeoDB::Search::Rating;

=cut

=head1 NAME

Rating - search waypoint ratings

=head1 SYNOPSIS

geoqo -s "rating:myrating>myvalue" -l

=head1 DESCRIPTION

START HELP

The rating plugin lets you search ratings that may have been placed on a cache using the modify I<rate> module.

  > geoqo -s ident:gcabcd -m rate:fun=8

  > geoqo -s "rating:fun>5" -d count
  Search Result Count: 1

If you have downloaded average ratings from the ratings server, you
can search by those as well by prefixing the name with "others:" to
search by the average rating given to the cache by others:

  > geoqo -s "rating:others:fun>5"

If you want to search for all caches that you have not rated for a
particular category, use:

  > geoqo -s "rating:fun=-"

END HELP

=head1 AUTHOR

Wes Hardaker (AKA Yamar) <hardaker ATAT users.sourceforge.net> 

=cut

use GeoDB::Utils;

use strict;

our @ISA = qw(GeoDB::Search);

our $VERSION = "0.71";

sub search {
    my ($self, $criteria, $type) = @_;

    my $clause = "";
    my @values;

    if ($self->{'optionsarray'}[0][2] eq '=-') {
	GEODEBUG(4, "rating query: treating as not exists");

	# XXX: broken for multiple queries.
	foreach my $opt (@{$self->{'optionsarray'}}) {
	    $clause = "waypoints.ident not in
                 (select ident from ratings where ratingname = ?)";
	    push @values, $opt->[0];
	}
	return ($clause, \@values);
    }

    foreach my $opt (@{$self->{'optionsarray'}}) {
	if ($opt->[0] =~ s/^others://) {
	    $clause .=
	      GeoDB::Search::get_search_subexpression("or ( ratingname ARG ? ",
						      '==', $opt->[0],
						      \@values);
	    $clause .=
	      GeoDB::Search::get_search_subexpression(" and othersrating ARG ? )",
						      $opt->[2], $opt->[1],
						      \@values);
	} else {
	    $clause .=
	      GeoDB::Search::get_search_subexpression("or ( ratingname ARG ? ",
						      '==', $opt->[0],
						      \@values);
	    $clause .=
	      GeoDB::Search::get_search_subexpression(" and ratingvalue ARG ? )",
						      $opt->[2], $opt->[1],
						      \@values);
	}
    }
    $clause =~ s/^or //;

    $clause =
      "waypoints.ident in (select ident from ratings where $clause)";

    GEODEBUG(4,"rating query: $clause\n");

    return  ($clause, \@values);
}

1;
