package GeoDB::Waypoint::Geodining;

use strict;
use GeoDB::Utils;
use GeoDB::Waypoint;

our $VERSION = "0.71";
our @ISA=qw(GeoDB::Waypoint);

# database handles
my ($getcacheh, $getidh, $newwayidh, $updatewh,
    $logdelh, $logaddh, $logselh, $logselfh);

# 'ident' not listed intentionally
our @geodining_dbcols = qw(
			   restaurant_name
			   submitted_by
			   price
			   service
			   continent
			   country
			   state
			   city
			   zip
			   address
			   description
			   details
			  );

# 'ident' not listed intentionally
our @geodining_class_cols = qw(
			       geodining_restaurant_name
			       geodining_submitted_by
			       geodining_price
			       geodining_class
			       geodining_continent
			       geodining_country
			       geodining_state
			       geodining_city
			       geodining_zip
			       geodining_address
			       geodining_description
			       geodining_details
			      );

our @geodining_waypoint_cols = (
			       @GeoDB::Waypoint::waypoints_dbcols,
			       @geodining_dbcols
			      );

sub init {
    my $dbh = shift;

    return if (!$dbh);

    # geodining handles
    ($getidh, $newwayidh, $updatewh) = 
      create_waypoint_handles($dbh, 'geodinings', \@geodining_dbcols);

    GeoDB::Waypoint::register_waypoint_type('Geodining',
					    'GeoDB::Waypoint::Geodining',
					    'geodinings',
					    \@geodining_dbcols,
					    \@geodining_class_cols);

    return 1;
}

# if it already exists in the db, return 1
sub typedata_exists {
    my ($self) = @_;
    $getidh->execute($self->{'data'}{'ident'});
    my $row = $getidh->fetchrow_arrayref();
    $getidh->finish();
    return 1 if $row;
    return 0;
}

sub _nohash {
    return "" if (ref($_[0]) eq 'HASH');
}

# save the current cache to the DB (either update or insert a new row)
sub save {
    my $self = shift;
    my $modgh = $newwayidh;
    # save to db

    $self->SUPER::save(@_);

    if ($self->typedata_exists()) {
	GEODEBUG(6,"updating geodining $self->{data}{ident}\n");
	$modgh = $updatewh;
    } else {
	GEODEBUG(6,"inserting geodining $self->{data}{ident}\n");
    }
    my $data = $self->{'data'};

    $modgh->execute(
		    $data->{'geodining_restaurant_name'},
		    $data->{'geodining_submitted_by'},
		    $data->{'geodining_price'},
		    $data->{'geodining_class'},
		    $data->{'geodining_continent'},
		    $data->{'geodining_country'},
		    $data->{'geodining_state'},
		    $data->{'geodining_city'},
		    $data->{'geodining_zip'},
		    $data->{'geodining_address'},
		    $data->{'geodining_description'},
		    $data->{'geodining_details'},
		    $data->{'ident'},
		  );
    if ($modgh->errstr) {
	GEOERR("Error saving $data->{ident}: " . $modgh->errstr . "\n");
    }
}

sub ssummary {
    my ($self, $count, $options) = @_;

    return 
      $self->sprint_summary_line($count, ($options->{'format'} ? "$options->{'format'}\n" : "%{3:count} %{8:ident} %{66:geodining_restaurant_name}\n"));
}

sub sdisplay {
    my ($self, $dologs) = @_;
    my $data = $self->{'data'};
    my $out;
    $out .= sprintf("%-8s   %-31s Location:  %s, %s\n",
	   $data->{'ident'}, $data->{'type'} . " / " . $data->{'subtype'},
	   sprint_coords($data->{'lat'}, $data->{'lon'}));
    $out .= sprintf("Restaurant:    %s\n", $self->{'data'}{'geodining_restaurant_name'});
    $out .= sprintf("Price/Class:   %s/%s\n",
	   $self->{'data'}{'geodining_price'},
	   $self->{'data'}{'geodining_class'});
    $out .= sprintf("Address:   %s\n", $data->{'geodining_address'});
    $out .= sprintf("	       %s, %s  %s\n",
	   $data->{'geodining_city'}, $data->{'geodining_state'},
	   $data->{'geodining_zip'});
    $out .= sprintf("	       %s\n", $data->{'geodining_country'});
    $out .= sprintf("Description:\n");
    $out .= format_output($data->{'geodining_description'});
    $out .= sprintf("Details:\n");
    $out .= format_output($data->{'geodining_details'});
    $out .= $self->sshow_attributes();
    $out .= $self->sshow_tags();
    $out .= $self->sshow_ratings();
    $out .= $self->sshow_logs() if ($dologs);
    return $out;
}

1;
