#!/usr/bin/perl

use strict;

my $outputfile = "./install_modules";

our @required = qw(
  XML::Parser
  XML::Simple
  DBI
  DBD::SQLite2
);

our %optional = (
		 'IO::String' =>
		 'Needed to use the online help system',

		 'HTML::FormatText' => 
		 'formats cache HTML descriptions to text',

		 'Text::Wrap' => 
		 'wraps text nicely on the screen',

		 'WWW::Babelfish' =>
		 'translate languages in cache fields to different languages',

		 'Term::ProgressBar' => 
		 'for getting a progress bar display as it performs tasks',

		 'Archive::Zip' =>
		 'for Zip import format to work',

		 'LWP::UserAgent' =>
		 'for publishing and fetching "tag"s to the tag server',
		);

our %gui =
  ('QWizard' =>
   'Question and Answer Wizard module',

   'Gtk2' =>
   'A Graphical front end.  Ideally you want this; but Tk will be ok too.',

   'Tk' =>
   'A Graphical front end.  You don\'t need both Gtk2 and Tk.  Pick one.',

   'Chart::Lines' =>
   'If you want to display graphs'
  );

our @classifiers =
  (
   'Lingua::PT::Stemmer',
   'Lingua::Stem::It',
   'Lingua::Stem::Snowball::No',
   'Text::German',
   'Lingua::Stem::Snowball::Se',
   'Lingua::Stem::Fr',
   'Lingua::GL::Stemmer',
   'Lingua::Stem::Snowball::Da',
   'Lingua::Stem::Ru',

   'Params::Validate',
   'Lingua::Stem',
   'Algorithm::NaiveBayes',
   'Class::Container',
   'Statistics::Contingency',

   'AI::DecisionTree',
   'Algorithm::SVM',

   'AI::Categorizer',
   'AI::Categorizer::Collection::Files',
   'AI::Categorizer::Collection::InMemory',
   'AI::Categorizer::Learner::NaiveBayes',
   'AI::Categorizer::Learner::DecisionTree',
   'AI::Categorizer::Learner::SVM',
   'File::Spec',
  );

our %classifiers;
map { $classifiers{$_} = 'Needed for doing smart cache classification.';
  } @classifiers;


our %groups = 
  ( 'AMiscellanious' => \%optional,
    'BGraphical User Interface' => \%gui,
    'CSmart Cache Classification' => \%classifiers,
  );

open(O,">" . $outputfile);
print O "#!/usr/bin/perl\n\nuse CPAN;\n\n";

my $havebasics = 1;
print "\nChecking required modules:\n";
foreach my $m (@required) {
    my $haveit = eval "require $m;";
    if ($haveit) {
	printf("    %-50s %s\n", $m, "   you have it");
    } else {
	$havebasics = 0;
	printf("    %-50s %s\n", $m, "** YOU NEED IT");
	print O "install \"$m\";\n";
    }
}

if (!$havebasics) {
    print "\nYou need at least the above modules to get GeoQO working\n";
    print "Once you have installed them, you can run this script to list\n";
    print "extra modules you may or may not want as well\n\n";

    print_instructions();
}

print "\nChecking optional modules:\n";

my $haveeverything = 1;
foreach my $name (sort keys(%groups)) {
    my $n = $name;
    $n =~ s/^.//;
    print "\n  $n:\n";
    my %x = %{$groups{$name}};
    foreach my $m (keys(%x)) {
	my $haveit = eval "require $m;";
	if ($haveit) {
	    printf("    %-50s %s\n", $m, "   you have it");
	} else {
	    printf("    %-50s %s\n      (%s)\n", $m, "** YOU MAY WANT IT", $x{$m});
	    print O "install \"$m\";\n";
	    $haveeverything = 0;
	}
    }
}
print "\n";
print_instructions($haveeverything);

sub print_instructions {
    my $x = shift;
    if (!$x) {
	print O "\n\nprint \"\\n\\nWell...  I did my best; rerun check-prerequisites.pl to double check things\\n\";\n";
	close(O);
	chmod 0755, $outputfile;
	print "I created a $outputfile for you.  To attempt to install
the above missing perl modules, run this file as follows:

  perl $outputfile

Good Luck!
";
    } else {
	print "CONGRATULATIONS!  You have everything you could ever hope for.\n\n";
	unlink($outputfile);
    }
    exit;
}

