# ==============================================================================
# Alfandega::Addons::check_spoofing.pm
# ------------------------------------------------------------------------------
# Copyright(c) 2002 Christian Tosta <c_tosta@yahoo.com.br>. All rights reserved.
# This program is free software; you can redistribute it and/or modify it under
# the same terms as Perl itself.
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
#
#                        Alfandega Check Spoofing Add-On
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

package Alfandega::Addons::CheckSpoofing;

require 5.004;
require Exporter;

use strict;
use vars qw(@ISA $VERSION %Config);

use Alfandega::Language;

$VERSION = "2.1";
@ISA     = qw(Exporter);

sub new {

    shift;
    my $file = shift;
    my $table = shift;
    my $class = shift;
    my ($i,$direction,$address,$rule1,$rule2,$log,$dump);
    my @AclValues;
    my %ACLs;
    my %Rules;
    my %Params;
    my %NewRules;
    my %ValidRules;
    %Config = $class->config;

    Alfandega::Language->new($Config{'language'});

    my %actions = (
        'limit'      =>      'allowed',
        'loglimit'   =>      'LOGallowed',
        'deny'       =>      'DROP',
        'logdeny'    =>      'LOGDROP',
        'reject'     =>      'REJECT',
        'logreject'  =>      'LOGREJECT',
        'return'     =>      'RETURN',
        'logreturn'  =>      'LOGRETURN',
        'logonly'    =>      'LOG',
    );

    return if $table !~ "filter";
    open(CFG,'<',$file);
    
    $i = "AAAA";

    while(<CFG>) {
        chomp;
        s/^\s*(.*?)\s*$/$1/;
        my @search = $_ if !/^#/ && !/^;/;

        foreach my $line (@search) {
            if($line) {
                my ($action,$status,$interface,$way,
                    $address,$rate,$burst,$level) = split(/\s+/,$line,8);

                $action = defined $actions{$action} ? $actions{$action} : undef;

                $direction = $status =~ "incoming" ? "-i" : undef;
                $direction = $status =~ "outcoming" ? "-o" : $direction;

                $interface =~ s/^\s*(.*?)\s*$/$1/;

                #warn locale("Spoofing::syntax_error") if !defined $action;
                #warn locale("Spoofing::syntax_error") if !defined $direction;
                #warn locale("Spoofing::syntax_error") if !defined $interface;
                #warn locale("Spoofing::syntax_error") if !defined $way;
                #warn locale("Spoofing::syntax_error") if !defined $address;

                $way = (defined $way && $way =~ "from") ? "-s" : $way;
                $way = (defined $way && $way =~ "to") ? "-d" : $way;

                $rate = (defined $rate) ? substr($rate,5) : $Config{'default_limit_rate'};
                $burst = (defined $burst) ? substr($burst,6) : $Config{'default_limit_burst'};
                $level = (defined $level) ? substr($level,6) : $Config{'default_log_level'};

                $log = defined $action && substr($action,0,3) =~ "LOG";
                $action = $log ? substr($action,3) : $action;

                my @params = ($address);

                ++$Params{$address};

                $rule1  = "-A chkSpoof $direction $interface $way $address ";
                $rule1 .= "-m limit --limit $rate --limit-burst $burst ";

                if($log) {
                    $rule2 = $rule1;
                    $rule1 .= "-j LOG --log-level $level --log-prefix \"Spoofed Packet: \" ";
                    $rule2 .= "-j $action";
                }
                else {
                    $rule1 .= "-j $action";
                    undef $rule2;
                }

                ++$Rules{$i++.":".$rule1} if $rule1;
                ++$Rules{$i++.":".$rule2} if $rule2;
            }
        }
    }

    foreach my $param (keys %Params) {
        $_ = $param if $param;
        my $isACL = /<<(\S+)>>/;
        $param =~ s/<<(\S+)>>/$1/ if $isACL;
        ++$ACLs{$param} if $isACL;
    }

    for(my $i=0;$i<10;$i++) {
        undef %NewRules;

        foreach my $rule (sort keys %Rules) {
            foreach my $acl (keys %ACLs) {
                if(defined $Config{$acl}) {
                    @AclValues = split(/,/,$Config{$acl});
                }
                else {
                    warn locale("Services::invalid_acl_name");
                    return;
                }

                foreach my $value (@AclValues) {
                    $_ = $rule;
                    s/<<$acl>>/$value/g;
                    ++$ValidRules{$_} if !/<<(\S+)>>/;
                    ++$NewRules{$_} if /<<(\S+)>>/;
                }
            }
        }
        %Rules = %NewRules;
    }

    foreach my $rule (sort keys %ValidRules) {
        $dump .= substr("$rule\n",5);
    }

    return $dump;
}

__END__