# ==============================================================================
# Alfandega::Addons::log_control.pm
# ------------------------------------------------------------------------------
# Copyright(c) 2002 Christian Tosta <c_tosta@yahoo.com.br>. All rights reserved.
# This program is free software; you can redistribute it and/or modify it under
# the same terms as Perl itself.
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
#
#                         Alfandega Log Control Add-On
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

package Alfandega::Addons::LogControl;

require 5.004;
require Exporter;

use strict;
use vars qw(@ISA $VERSION %Config);

$VERSION = "1.3";
@ISA     = qw(Exporter);


sub new {

    shift;
    my $file = shift;
    my $table = shift;
    my $class = shift;
    my @rules;
    my $rule;
    my $flags;
    my $rtype;
    my $dump;
    %Config = $class->config;

    return if $table !~ "filter";

    open(CFG,'<',$file);
    while(<CFG>) {
        chomp;
        s/^\s*(.*?)\s*$/$1/;
        my @search = $_ if !/^#/ && !/^;/;

        foreach my $line (@search) {
            if($line) {
                my ($target,$proto,$type,$level,
                    $prefix,$limit,$burst,$options) = split(/\s+/,$line,8);

                $level = defined $level ? substr($level,6) : undef;
                $prefix = defined $prefix ? substr($prefix,7) : undef;
                $limit = defined $limit ? substr($limit,6) : undef;
                $burst = defined $burst ? substr($burst,6) : undef;

                if (defined $options) {
                    foreach my $opt (split(/\|/,$options)) {
                        $flags = substr($opt,0,5) =~ "flags" ?
                        substr($opt,6) : $opt;
                        $rtype = substr($opt,0,11) =~ "reject-type" ?
                        substr($opt,12) : $opt;
                    }
                }

                $target = $target =~ "logdrop" ? "DROP" : $target;
                $target = $target =~ "logreject" ? "REJECT --reject-with $rtype" : $target;
                $target = $target =~ "loglimit" ? "ACCEPT" : $target;

                $rule = "-A " . $type;

                $type = $type =~ "fragment" ? "-f" : $type;
                $type = $type =~ "invalid" ? "-m state --state INVALID" : $type;
                $type = $type =~ "new" ? "-p $proto -m $proto --$proto-flags $flags -m " .
                "state --state NEW" : $type;
                $type = $type =~ "unclean" ? "-m unclean" : $type;

                $level = defined $level ? $level : $Config{'default_log_level'};
                $prefix = defined $prefix ? "--log-prefix $prefix" : undef;
                $limit = defined $limit ? $limit : $Config{'default_limit_rate'};
                $burst = defined $burst ? $burst : $Config{'default_limit_burst'};

                $level = "--log-level $level";
                $limit = "--limit $limit";
                $burst = "--limit-burst $burst";

                my $rule1 = "$rule $type -m limit $limit $burst -j LOG $level $prefix";

                my $rule2 = "$rule $type ";
                $rule2 .= "-m limit $limit $burst " if $target =~ "ACCEPT";
                $rule2 .= "-j $target";

                push(@rules,"$rule1\n$rule2");
            }
        }
    }
    foreach my $rule (@rules) {
        $dump .= "$rule\n";
    }
    return $dump;
}

__END__
