# ==============================================================================
# Alfandega::Addons::packet_forwarding.pm
# ------------------------------------------------------------------------------
# Copyright(c) 2002 Christian Tosta <c_tosta@yahoo.com.br>. All rights reserved.
# This program is free software; you can redistribute it and/or modify it under
# the same terms as Perl itself.
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
#
#                       Alfandega Packet Forwarder Add-On
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

package Alfandega::Addons::PacketForwarding;

require 5.004;
require Exporter;

use strict;
use vars qw(@ISA $VERSION);

$VERSION = "1.3";
@ISA     = qw(Exporter);


sub new {

    shift;
    my $file = shift;
    my $table = shift;
    my @rules;
    my $rule;
    my $dump;

    return if $table !~ "nat";

    open(CFG,'<',$file);
    while(<CFG>) {
        chomp;
        s/^\s*(.*?)\s*$/$1/;
        my @search = $_ if !/^#/ && !/^;/;

        foreach my $line (@search) {
            if($line) {
                $line = substr($line,8);
                my ($proto,$src_direction,$src_target,
                    $src_target_port,$dst_direction,$dst_target,
                    $dst_target_port) = split(/\s+/,$line,7);

                if($src_direction =~ $dst_direction && $src_direction =~ 'from') {
                    $rule = "-A POSTROUTING -p $proto -m $proto -s $src_target ";
                    if($src_target_port !~ "all-ports") {
                        $rule .= "--sport $src_target_port ";
                    }
                    $rule .= "-j SNAT --to-source $dst_target";
                    $rule .= ":$dst_target_port" if $dst_target_port !~ "all-ports";
                }
                elsif($src_direction =~ $dst_direction && $src_direction =~ 'to') {
                    $rule = "-A PREROUTING -p $proto -m $proto -d $src_target ";
                    if($src_target_port !~ "all-ports") {
                        $rule .= "--dport $src_target_port ";
                    }
                    $rule .= "-j DNAT --to-destination $dst_target";
                    $rule .= ":$dst_target_port" if $dst_target_port !~ "all-ports";
                }
                else { warn "Config error in file."; return 1; }

                push(@rules,$rule);

            }
        }
    }
    foreach my $rule (@rules) {
        $dump .= "$rule\n";
    }
    return $dump;
}

__END__
