# ==============================================================================
# Alfandega::Addons::remote_blacklist.pm
# ------------------------------------------------------------------------------
# Copyright(c) 2002 Christian Tosta <c_tosta@yahoo.com.br>. All rights reserved.
# This program is free software; you can redistribute it and/or modify it under
# the same terms as Perl itself.
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
#
#                       Alfandega Remote Blacklist Add-On
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

package Alfandega::Addons::RemoteBlacklist;

require 5.004;
require Exporter;

#use strict;
use vars qw(@ISA $VERSION);
use Compress::Zlib;
use Digest::MD5;
use Net::HTTP;
use Term::ProgressBar;
use constant MAX => 10_000_000;

$VERSION = "2.0";
@ISA     = qw(Exporter);


sub new {

    shift;
    my $class = shift;
    my $table = shift;
    my ($local_md5sum,$remote_md5sum,$rule,$f,@rules);
    my ($file,$md5,$dump);

    return if $table !~ "filter";

    my $server = "alfandega.sourceforge.net";
    my $remote_file_md5 = "/blacklist/blacklist.conf.gz.md5";
    my $spool_dir = "/var/spool/alfandega";
    my $local_file = $spool_dir . "/blacklist.conf.gz";
    my $remote_file = "/blacklist/blacklist.conf.gz";

    my $host = Net::HTTP->new(Host => $server) || die $@;
    $host->write_request(GET => $remote_file_md5,'User-Agent' => "Mozilla/5.0");

    my($code, $mess, %h) = $host->read_response_headers;

    while (1) {
        my $buf;
        my $n = $host->read_entity_body($buf, 1024);
        last unless $n;
        $remote_md5sum .= $buf;
    }

    open(FILE,'<',$local_file) or open(FILE,'>',$local_file);
    binmode(FILE);

    $md5 = Digest::MD5->new;
    while (<FILE>) {
        $md5->add($_);
    }
    close(FILE);
    $local_md5sum = $md5->hexdigest;

    ($remote_md5sum,$f) = split(/\s+/,$remote_md5sum,2);

    if(! -e $local_file || $local_md5sum !~ $remote_md5sum) {
        mkdir $spool_dir if ! opendir(DIR,$spool_dir);
        open(FILE,'>',$local_file)
        or die "Cannot open file: $local_file\n";

        binmode FILE;

        my $host = Net::HTTP->new(Host => $server) || die $@;
        $host->write_request(GET => $remote_file,
        'User-Agent' => "Mozilla/5.0");

        my($code, $mess, %h) = $host->read_response_headers;

        print "Download: http://$server $remote_file ";
        print "[$h{'Content-Length'} $h{'Accept-Ranges'}]\n";

        my $message = "blacklist.conf.gz [$h{'Content-Length'} $h{'Accept-Ranges'}]";

        my $max = int($h{'Content-Length'}+0) || MAX;
        my $progress = Term::ProgressBar->new({name => $message, count => $max,
                                               ETA => 'linear', });
        $progress->max_update_rate(1);
        my $bytes = 0;

        for (0..$max) {
        while (1) {
            my $buf;
            my $n = $host->read_entity_body($buf, 1024);
            last unless $n;
            my $bufsize = length "$buf";
            $bytes = $progress->update($bufsize) if $bufsize > $bytes;
            syswrite(FILE,$buf) or die "error writing file.\n";
        }
        }
        $progress->update($max) if $max >= $bytes;
        close(FILE);
    }

    my ($line,@buffer);
    my $gz = gzopen($local_file,"rb")
    or die "Cannot open $local_file: $gzerrno\n";
    while ($gz->gzreadline($_) > 0) {
        push(@buffer,$_) if !/^#/ && !/^;/;
    }
    die "Error reading from $local_file: $gzerrno\n" if $gzerrno != Z_STREAM_END;
    $gz->gzclose();

        foreach my $line (@buffer) {
            if($line) {
                $line = substr($line,0,-1);
                
                my @params = split(/\t/,$line,2);
                my $mac = index($line,":") <= -1 ? undef : $params[0];
                my $ip4 = index($line,":") <= -1 ? $params[0] : undef;

                $rule = "-I FORWARD -m mac --mac-source $mac -j DROP" if $mac;
                $rule = "-A blacklist -s $ip4 -j DROP" if $ip4;

                push(@rules,$rule);

            }
        }
    
    foreach my $rule (@rules) {
        $dump .= "$rule\n";
    }
    return $dump;
}

__END__
