# ==============================================================================
# Alfandega::Addons::forward_services.pm
# ------------------------------------------------------------------------------
# Copyright(c) 2002 Christian Tosta <c_tosta@yahoo.com.br>. All rights reserved.
# This program is free software; you can redistribute it and/or modify it under
# the same terms as Perl itself.
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
#
#                       Alfandega Forward Services
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

package Alfandega::Addons::Services;

require 5.004;
require Exporter;

use strict;
use vars qw(@ISA $VERSION %Config);

use Alfandega::Language;

$VERSION = "0.3";
@ISA     = qw(Exporter);


sub new {

    shift;
    my $file = shift;
    my $table = shift;
    my $class = shift;
    my ($chain,$param,$rule,$dump);
    my @AclValues;
    my %ACLs;
    my %Rules;
    my %Params;
    my %NewRules;
    my %ValidRules;
    %Config = $class->config;

    Alfandega::Language->new($Config{'language'});

    my %actions = (
        'allow'      =>      'allowed',
        'deny'       =>      'DROP',
        'reject'     =>      'REJECT',
        'return'     =>      'RETURN',
        'log'        =>      'LOG',
    );

    my %protocols = (
        'icmp'       =>      '1',
        'tcp'        =>      '2',
        'udp'        =>      '3',
    );

    return if $table !~ "filter";
    open(CFG,'<',$file);

    while(<CFG>) {
        chomp;
        s/^\s*(.*?)\s*$/$1/;
        my @search = $_ if !/^#/ && !/^;/;

        foreach my $line (@search) {
            if($line) {
                my ($action,$proto,$status,$crossway,$way1,
                    $param1,$ports1,$way2,$param2,$ports2,
                    $rate,$burst) = split(/\s+/,$line,10);

                $action = defined $actions{$action} ? $actions{$action} : undef;

                $proto = defined $protocols{$proto} ? $proto : undef;

                $chain = $status =~ "incoming" ? $proto . "in" : undef;
                $chain = $status =~ "outcoming" ? $proto . "out" : $chain;
                $chain = $status =~ "crossing" ? "chkForwd" : $chain;

                $crossway =~ s/any//g;
                $crossway =~ s/(\S+)->/-i $1->/;
                $crossway =~ s/->(\S+)/-> -o $1/;
                $crossway =~ s/->//g;
                $crossway =~ s/^\s*(.*?)\s*$/$1/;

                warn locale("Services::syntax_error") if !defined $action;
                warn locale("Services::syntax_error") if !defined $proto;
                warn locale("Services::syntax_error") if !defined $chain;
                warn locale("Services::syntax_error") if !defined $crossway;
                warn locale("Services::syntax_error") if !defined $way1;
                warn locale("Services::syntax_error") if !defined $param1;
                warn locale("Services::syntax_error") if !defined $ports1;

                $way1 = (defined $way1 && $way1 =~ "from") ? "-s $param1 --sport" : $way1;
                $way1 = (defined $way1 && $way1 =~ "to") ? "-d $param1 --dport" : $way1;

                $way2 = (defined $way2 && $way2 =~ "from") ? "-s $param2 --sport" : $way2;
                $way2 = (defined $way2 && $way2 =~ "to") ? "-d $param2 --dport" : undef;

                warn locale("Services::syntax_error") if (defined $way2 && (!defined $param2 || !defined $ports2));
                warn locale("Services::syntax_error") if (defined $param2 && (!defined $way2 || !defined $ports2));
                warn locale("Services::syntax_error") if (defined $ports2 && (!defined $param2 || !defined $way2));

                my @params = ($param1,$param2);
                my @ports = ($ports1,$ports2);

                ++$Params{$param1};
                ++$Params{$param2} if $param2;
                ++$Params{$ports1};
                ++$Params{$ports2} if $ports2;

                $rule = "-A $chain " if $chain;
                $rule .= "-p $proto -m $proto " if $proto;
                $rule .= "$crossway " if $crossway;
                $rule .= "$way1 $ports1 " if ($way1 && $ports1);
                $rule .= "$way2 $ports2 " if ($way2 && $ports2);
                $rule .= "-j $action" if $action;

                ++$Rules{$rule} if $rule;
            }
        }
    }

    foreach my $param (keys %Params) {
        $_ = $param if $param;
        my $isACL = /<<(\S+)>>/;
        $param =~ s/<<(\S+)>>/$1/ if $isACL;
        ++$ACLs{$param} if $isACL;
    }

    for(my $i=0;$i<10;$i++) {
        undef %NewRules;

        foreach $rule (keys %Rules) {
            foreach my $acl (keys %ACLs) {

                if(defined $Config{$acl}) {
                    @AclValues = split(/,/,$Config{$acl});
                }
                else {
                    warn locale("Services::invalid_acl_name");
                    return;
                }

                foreach my $value (@AclValues) {
                    $_ = $rule;
                    s/<<$acl>>/$value/g;
                    ++$ValidRules{$_} if !/<<(\S+)>>/;
                    ++$NewRules{$_} if /<<(\S+)>>/;
                }
            }
        }
        %Rules = %NewRules;
    }

    foreach $rule (keys %ValidRules) {
        $dump .= "$rule\n";
    }
    
    return $dump;
}

__END__