# ==============================================================================
# Alfandega::Control.pm
# ------------------------------------------------------------------------------
# Copyright(c) 2002 Christian Tosta <c_tosta@yahoo.com.br>. All rights reserved.
# This program is free software; you can redistribute it and/or modify it under
# the same terms as Perl itself.
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
#
#                               Iptables Control
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

package Alfandega::Control;

require 5.004;
require Exporter;

use strict;
use vars qw(@ISA @EXPORT $VERSION %Config);
use Alfandega::Language;
use Alfandega::Modules;
use Alfandega::SysCtl;

@ISA = qw(Exporter);
@EXPORT = qw(clean panic save start status stop);
$VERSION = "2.1";

sub clean { 

    my $class = shift;
    %Config = $class->config;

    unlink $Config{'iptables_dump_file'};
}

sub checktables {

    shift;
    my $class = shift;
    %Config = $class->config;

    Alfandega::Language->new($Config{'language'});

    # Initialize necessary tables
    restore($class->createtable("filter"),"-n");

    if(-x $Config{'config_basedir'} . "/addons/packet_forwanding.conf") {
        restore($class->createtable("nat"),"-n");
    }
}

# We want to test if table exists in netfilter before create dumps.

sub iftable {

    my @tables = @_;

    my %ok = ();
    my $search = 'while (<>) { study;';
    foreach my $table (@tables) {
        $search .= "++\$ok{$table} if /\\b$table\\b/;\n";
    }
    $search .= "}";
    @ARGV = ("/proc/net/ip_tables_names");
    undef $/;
    eval $search;
    $/ = "\n";
    my @matches = ();
    foreach my $table (sort keys(%ok)) {
        push(@matches,$table);
    }
    return @matches;
}

# This will tells to netfilter to block all packets.

sub panic {

    my @filter = iftable("filter");
    my @mangle = iftable("mangle");
    my @nat    = iftable("nat");

    my $filter = shift @filter;
    my $mangle = shift @mangle;
    my $nat    = shift @nat;

    my $fwdump .= "*$filter\n" if $filter;
       $fwdump .= ":INPUT DROP [0:0]\n" if $filter;
       $fwdump .= ":FORWARD DROP [0:0]\n" if $filter;
       $fwdump .= ":OUTPUT DROP [0:0]\n" if $filter;
       $fwdump .= "COMMIT\n" if $filter;
       $fwdump .= "*$mangle\n" if $mangle;
       $fwdump .= ":INPUT DROP [0:0]\n" if $mangle;
       $fwdump .= ":FORWARD DROP [0:0]\n" if $mangle;
       $fwdump .= ":OUTPUT DROP [0:0]\n" if $mangle;
       $fwdump .= ":PREROUTING DROP [0:0]\n" if $mangle;
       $fwdump .= ":POSTROUTING DROP [0:0]\n" if $mangle;
       $fwdump .= "COMMIT\n" if $mangle;
       $fwdump .= "*$nat\n" if $nat;
       $fwdump .= ":OUTPUT DROP [0:0]\n" if $nat;
       $fwdump .= ":PREROUTING DROP [0:0]\n" if $nat;
       $fwdump .= ":POSTROUTING DROP [0:0]\n" if $nat;
       $fwdump .= "COMMIT\n" if $nat;

    return restore($fwdump);
}

# Restores firewall from a netfilter configuration stream.

sub restore {

    my $dump = shift;
    my $options = shift;

    $options = "" unless $options;

    my $cmd = "echo -ne '$dump'| iptables-restore $options";
    system($cmd);
    die locale("Control::restore")."\n$dump\n" if $?;
    return $?;
}

sub save {

    my $class = shift;
    %Config = $class->config;

    my $cmd = "iptables-save > " . $Config{'iptables_dump_file'};
    system("$cmd");
    return $?;
}

sub start {

    my $class = shift;
    my @addons;
    my @core;
    my $dump;
    my $rule;
    my $table;

    stop();    # Be sure the firewall stopped after start it

    # Run External programs like scripts to authenticate to ADSL provider
    require $Config{'external_scripts'};

    Alfandega::Modules->load_modules($class);
    Alfandega::SysCtl->sysctl($class); # Kernel Tuning

    if(-e $Config{'iptables_dump_file'}) {
        open(DUMP,'<'.$Config{'iptables_dump_file'});
        while(<DUMP>) {
            chomp;
            $dump .= $_ . "\n";
        }
        print "Restoring Rules";
        restore($dump);
        exit 0;
    }

    # Generate the rules and import it

    foreach $table (iftable("filter","mangle","nat")) {
        undef $dump;
        push(@core,"*$table");
        push(@core,$class->createchains($table));
        push(@core,$class->createrules("core",$table));
        push(@core,"COMMIT");
    }

    foreach $rule (@core) {
        $dump .= "$rule\n";
    }

    $dump = $class->evaluate($dump);
    restore($dump);

    foreach $table (iftable("filter","mangle","nat")) {
        undef $dump;
        push(@addons,"*$table");
        push(@addons,$class->createrules("addons","$table"));
        if($table =~ "filter") {
            push(@addons,$class->remote("remote_blacklist","$table"));
        }
        push(@addons,"COMMIT");
    }

    foreach $rule (@addons) {
        $dump .= "$rule\n" if $rule;
    }

    $dump = $class->evaluate($dump);
    restore($dump,"-n");
}

sub status {

    my @tables = iftable("filter","mangle","nat");
    my $hr  = "----------------------------------------";
    my $dhr = "========================================";

    print "$dhr$dhr\n";
    foreach my $table (@tables) {
        print locale("Control::status::table"),"$table\n";
        print "$hr$hr\n";
        my $status = `iptables -t $table --list -v -n`;
        my %translated = (
            'policy'        => locale("Control::status::policy"),
            'packets'       => locale("Control::status::packets"),
            'pkts'          => locale("Control::status::pkts"),
            'in'            => locale("Control::status::in"),
            'out'           => locale("Control::status::out"),
            'source'        => locale("Control::status::source"),
            'destination'   => locale("Control::status::destination"),
            'opt'           => locale("Control::status::opt"),
            'target'        => locale("Control::status::target"),
            'references'    => locale("Control::status::references"),
        );
        $status =~ s/policy/$translated{'policy'}/g;
        $status =~ s/packets/$translated{'packets'}/g;
        $status =~ s/pkts/$translated{'pkts'}/g;
        $status =~ s/target/$translated{'target'}/g;
        $status =~ s/ in / $translated{'in'} /g;
        $status =~ s/ out / $translated{'out'} /g;
        $status =~ s/source/$translated{'source'}/g;
        $status =~ s/destination/$translated{'destination'}/g;
        $status =~ s/ opt/ $translated{'opt'}/g;
        $status =~ s/references/$translated{'references'}/g;
        print "$status\n";
        print "$dhr$dhr\n";
    }

    if($Config{'show_conntrack_info'} =~ "on") {
        print locale("Control::status::conntrack");
        print "$hr$hr\n";
        my $conntrack = "/proc/net/ip_conntrack";

        if(open(CONNTRACK,'<',$conntrack)) {
            print <CONNTRACK>;
	    close(CONNTRACK);
        }

        print "$dhr$dhr\n";
    }

    print locale("Control::status::modules");
    print "$hr$hr\n";
    print locale("Control::status::modbanner");
    print "$hr\n";
    open(MODULES,'<',"/proc/modules") or warn locale("Control::modules"),$!;
    while (<MODULES>) {
        chomp;
        my ($mod,$size,$used,$dep) = s/^\s*(.*?)\s*$/$1/ &&
        split(/\s+/,$_,4) if /ip/;
        next if !$mod;
        while (length($mod) < 20 && length($mod) > 0) {
            $mod .= " ";
        }
        $dep = defined $dep ? $dep : "";
        print "$mod$dep\n" if $mod;
    }
    close(MODULES);
    print "$dhr$dhr\n";
    return $?;
}


# This will tells to netfilter to accept all packets. This is the factory
# netfilter settings.

sub stop {

    Alfandega::Modules->unload_modules();

    my @filter = iftable("filter");
    my @mangle = iftable("mangle");
    my @nat    = iftable("nat");

    my $filter = shift @filter;
    my $mangle = shift @mangle;
    my $nat    = shift @nat;

    my $fwdump .= "*$filter\n" if $filter;
       $fwdump .= ":INPUT ACCEPT [0:0]\n" if $filter;
       $fwdump .= ":FORWARD ACCEPT [0:0]\n" if $filter;
       $fwdump .= ":OUTPUT ACCEPT [0:0]\n" if $filter;
       $fwdump .= "COMMIT\n" if $filter;
       $fwdump .= "*$mangle\n" if $mangle;
       $fwdump .= ":INPUT ACCEPT [0:0]\n" if $mangle;
       $fwdump .= ":FORWARD ACCEPT [0:0]\n" if $mangle;
       $fwdump .= ":OUTPUT ACCEPT [0:0]\n" if $mangle;
       $fwdump .= ":PREROUTING ACCEPT [0:0]\n" if $mangle;
       $fwdump .= ":POSTROUTING ACCEPT [0:0]\n" if $mangle;
       $fwdump .= "COMMIT\n" if $mangle;
       $fwdump .= "*$nat\n" if $nat;
       $fwdump .= ":OUTPUT ACCEPT [0:0]\n" if $nat;
       $fwdump .= ":PREROUTING ACCEPT [0:0]\n" if $nat;
       $fwdump .= ":POSTROUTING ACCEPT [0:0]\n" if $nat;
       $fwdump .= "COMMIT\n" if $nat;

    return restore($fwdump);
}

sub run { return `@_`; }

__END__
