# ==============================================================================
# Alfandega::Firewall.pm
# ------------------------------------------------------------------------------
# Copyright(c) 2002 Christian Tosta <c_tosta@yahoo.com.br>. All rights reserved.
# This program is free software; you can redistribute it and/or modify it under
# the same terms as Perl itself.
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
#
#                             Alfandega Main Class
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

package Alfandega::Firewall;

require 5.004;

use strict;
use vars qw(@ISA @EXPORT $VERSION %Config);

use Alfandega::Addons;
use Alfandega::Config;
use Alfandega::Control;
use Alfandega::Interfaces;
use Alfandega::Language;

$VERSION = "2.1";
@EXPORT = qw(%Config);
@ISA = qw(Exporter);

sub new {

    my $class = shift;
    my $cfgfile = shift;
    my $self = {};
    bless $self,$class;

    %Config = Alfandega::Config->new($cfgfile);
    Alfandega::Control->checktables($class);
    Alfandega::Language->new($Config{'language'});

    return $self;
}

sub config {
    return %Config;
}

sub createchains {

    shift;
    my $table = shift;
    my @rules;

    if($table =~ "filter") {
        push(@rules,":INPUT ".$Config{'default_input_policy'}." [0:0]");
        push(@rules,":FORWARD ".$Config{'default_forward_policy'}." [0:0]");
        push(@rules,":OUTPUT ".$Config{'default_output_policy'}." [0:0]");
    }
    
    if($table =~ "nat") {
        push(@rules,":PREROUTING ".$Config{'default_nat_prerouting_policy'}." [0:0]");
        push(@rules,":POSTROUTING ".$Config{'default_nat_postrouting_policy'}." [0:0]");
        push(@rules,":OUTPUT ".$Config{'default_nat_output_policy'}." [0:0]");
    }

    if($table =~ "mangle") {
        push(@rules,":INPUT ".$Config{'default_input_policy'}." [0:0]");
        push(@rules,":FORWARD ".$Config{'default_forward_policy'}." [0:0]");
        push(@rules,":OUTPUT ".$Config{'default_output_policy'}." [0:0]");
        push(@rules,":PREROUTING ".$Config{'default_forward_policy'}." [0:0]");
        push(@rules,":POSTROUTING ".$Config{'default_output_policy'}." [0:0]");
    }

    open(CHAINS,'<',$Config{'config_basedir'} . "/chains.conf")
    or die locale("Firewall::chains"),$!;
    while(<CHAINS>) {
        chomp;
        my ($chain) = s/^\s*(.*?)\s*$/$1/ &&
        split(/\s+/,$_) if /$table/ && !/^#/;
        push(@rules,":".$chain." - [0:0]") if $chain;
    }
    close(CHAINS);
    return @rules;
}

sub createrules {

    my $class = shift;
    my $target = shift;
    my $table = shift;
    my @files;
    my @rules;
    my $rule;

    chdir $Config{'config_basedir'} . "/core/" . $Config{'server_type'} ||
    die locale("Firewall::chdir_core"), $! if $target =~ "core";

    chdir $Config{'config_basedir'} . "/addons" ||
    die locale("Firewall::chdir_addons"), $! if $target =~ "addons";

    do {
        opendir(DIR, '.') || die locale("Firewall::open_dir"),$target,$!;
        while (my $file = readdir DIR) {
            next if $file =~ /^\./;
            push @files,$file if -f $file;
        }
        sleep 300 unless @files;
    } until @files;

    foreach my $file (@files) {
        #next if $file =~ "logpolicy.conf" &&
        #$Config{'log_default_policy'} =~ "off";
        next if $file =~ "logpolicy.conf";
        next if $file =~ "README";

        if($target =~ "core") {

            return if $table !~ "filter";

            open(CFG,'<',$file);
            while(<CFG>) {
                chomp;
                s/^\s*(.*?)\s*$/$1/;
                my @search = $_ if !/^#/ && /\|/;
                foreach my $line (@search) {
                    my ($chain,$bparams,$proto,$match,$mparams,
                        $target,$tparams) = split(/\|/,$line,7);

                    $rule  = "-A $chain";
                    $rule .= " $bparams" if $bparams;
                    $rule .= " -p $proto" if $proto;
                    $rule .= " -m $match" if $match;
                    $rule .= " $mparams" if $mparams;
                    $rule .= " -j $target" if $target;
                    $rule .= " $tparams" if $tparams;
                    push(@rules,$rule) if $line;
                }
            }
        }
        elsif($target =~ "addons") {
            next if ! -x $file;
            my $addon = substr($file,0,-5);
            my @params = ($file,$table,$class);
            $rule = Alfandega::Addons->load($addon,@params);
            push(@rules,$rule) ||
            die locale("Firewall::load_addon"),$file,$! if $target =~ "addon";
        }
    }
    foreach my $file (@files) {
        
        next if $file !~ "logpolicy.conf";
        next if $Config{'log_default_policy'} =~ "off";

        if($target =~ "core") {

            return if $table !~ "filter";

            open(CFG,'<',$file);
            while(<CFG>) {
                chomp;
                s/^\s*(.*?)\s*$/$1/;
                my @search = $_ if !/^#/ && /\|/;
                foreach my $line (@search) {
                    my ($chain,$bparams,$proto,$match,$mparams,
                        $target,$tparams) = split(/\|/,$line,7);

                    $rule  = "-A $chain";
                    $rule .= " $bparams" if $bparams;
                    $rule .= " -p $proto" if $proto;
                    $rule .= " -m $match" if $match;
                    $rule .= " $mparams" if $mparams;
                    $rule .= " -j $target" if $target;
                    $rule .= " $tparams" if $tparams;
                    push(@rules,$rule) if $line;
                }
            }
        }
    }
    return @rules;
}

sub createtable {

    shift;
    my $table = shift;
    my $dump = "*$table\nCOMMIT\n";
    return $dump;
}

sub evaluate {

    my $class = shift;
    my $dump = shift;

    %Config = ifconfig($class);

    while (my ($key,$value) = each %Config) {
        $dump =~ s/\<$key\>/$value/g;
    }
    return $dump;
}

sub remote {

    shift;
    my $addon = shift;
    my $table = shift;
    my @rules;

    if( -x $Config{'config_basedir'} . "/addons/$addon") {
        my @remote_rules = eval("Alfandega::Addons::$addon->new($table)");
        @rules = (@rules,@remote_rules);
        return @rules;
    }
    return;
}

__END__
