# ==============================================================================
# Alfandega::Interfaces.pm
# ------------------------------------------------------------------------------
# Copyright(c) 2002 Christian Tosta <c_tosta@yahoo.com.br>. All rights reserved.
# This program is free software; you can redistribute it and/or modify it under
# the same terms as Perl itself.
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
#
#                            Firewall Box Interfaces
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

package Alfandega::Interfaces;

require 5.004;

use strict;
use vars qw(@ISA @EXPORT $VERSION %Config);
use IO::Socket;
use IO::Interface qw(:flags);
use Alfandega::Language;

@ISA = ('Exporter');
@EXPORT = qw(ifconfig);
$VERSION = "2.0";

sub ifconfig {

   my $class = shift;
   %Config = $class->config;

   Alfandega::Language->new($Config{'language'});

   my $socket = IO::Socket::INET->new(Proto => 'udp');
   my @interfaces = $socket->if_list;
   my @fw_ifaces;

   push(@fw_ifaces,$Config{'loopback_interface'});
   push(@fw_ifaces,$Config{'internal_interface'});
   push(@fw_ifaces,$Config{'external_interface'});
   #push(@fw_ifaces,$Config{'dmz_interface'});
   #push(@fw_ifaces,$Config{'vpn_interface'});

   for my $fw_if (@fw_ifaces) {
       my $dev_found = 0;
       my $flags = $socket->if_flags($fw_if);

       for my $ac_if (@interfaces) {
           #next if substr($ac_if,0,2) =~ "lo";
           $dev_found |= $ac_if =~ $fw_if ? 1 : 0;
       }
       die locale("Interfaces::not_configured") . $fw_if unless $dev_found;
       if($Config{'ip_dynaddr'} =~ "off" and $flags & IFF_POINTOPOINT) {
           die locale("Interfaces::oops_dynamic") . $fw_if;
       }
   }

   if($Config{'ip_dynaddr'} =~ "on") {
       open(CRONTAB,'>/etc/cron.hourly/alfandega');
       syswrite(CRONTAB,"#!/bin/sh\n/sbin/alfandega start > /dev/null 2>&1");
       chmod 0755, "/etc/cron.hourly/alfandega";
   }
   else { unlink "/etc/cron.hourly/alfandega"; }

   for my $if (@interfaces) {
       #next if substr($if,0,2) =~ "lo";

       my $addr = $socket->if_addr($if);
       my $netmask = $socket->if_netmask($if);
       my $net = "$addr/$netmask";

       foreach my $var (keys %Config) {
           my $basename = substr($var,0,index($var,"_"));
           $Config{$basename . '_address'} = $addr if $Config{$var} =~ $if;
           $Config{$basename . '_network'} = $net if $Config{$var} =~ $if;
       }
   }
   return %Config;
}

__END__
