# ==============================================================================
# Alfandega::Modules.pm
# ------------------------------------------------------------------------------
# Copyright(c) 2002 Christian Tosta <c_tosta@yahoo.com.br>. All rights reserved.
# This program is free software; you can redistribute it and/or modify it under
# the same terms as Perl itself.
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
#
#                       Netfilter Kernel Modules Control
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

package Alfandega::Modules;

require 5.004;
require Exporter;

use strict;
use vars qw(@ISA $VERSION %Config);

$VERSION = "1.3";
@ISA     = qw(Exporter);

sub load_modules {

    shift;
    my $class = shift;
    %Config = $class->config;

    my $ftp_conntrack_support = $Config{'conntrack_ftp_support'};
    my $irc_conntrack_support = $Config{'conntrack_irc_support'};
    my $ftp_nat_support = $Config{'nat_ftp_support'};
    my $irc_nat_support = $Config{'nat_irc_support'};

    system("modprobe ip_conntrack_ftp") if $ftp_conntrack_support =~ "on";
    system("modprobe ip_nat_ftp") if $ftp_nat_support =~ "on";
    system("modprobe ip_conntrack_irc") if $irc_conntrack_support =~ "on";
    system("modprobe ip_nat_irc") if $irc_nat_support =~ "on";

    return if $Config{'load_kernel_modules'} =~ "off";

    open(KMOD,'<',$Config{'config_basedir'} . "/modules.conf");
    while(<KMOD>) {
        chomp;
        my @modules = $_ if !/^#/;
        foreach my $module (@modules) {
            my $length = length($module);
            $module = substr($module,0,-1) if substr($module,$length,1) =~ "\n";
            system("modprobe $module") if $module;
        }
    }
}

sub unload_modules {

    system("rmmod -a");

}

__END__
