# ==============================================================================
# Alfandega::SysCtl.pm
# ------------------------------------------------------------------------------
# Copyright(c) 2002 Christian Tosta <c_tosta@yahoo.com.br>. All rights reserved.
# This program is free software; you can redistribute it and/or modify it under
# the same terms as Perl itself.
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+
#
#                         Kernel Ipv4 Subsystem Control
#
# =+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+=+

package Alfandega::SysCtl;

require 5.004;
require Exporter;

use strict;
use vars qw(@ISA @EXPORT $VERSION %Config);
use Alfandega::Language;
use Alfandega::Modules;

@ISA = qw(Exporter);
@EXPORT = qw(sysctl);
$VERSION = "1.3";

sub sysctl {

    shift;
    my $class = shift;
    %Config = $class->config;

    my @sysctl;
    my %params = (
        'ip_dynaddr'                         => 'net.ipv4',
        'ip_forward'                         => 'net.ipv4',
        'icmp_ignore_bogus_error_responses'  => 'net.ipv4',
        'tcp_syncookies'                     => 'net.ipv4',
        'icmp_echo_ignore_broadcasts'        => 'net.ipv4',
        'icmp_echo_ignore_all'               => 'net.ipv4',
        'tcp_timestamps'                     => 'net.ipv4',
        'tcp_fin_timeout'                    => 'net.ipv4',
        'tcp_keepalive_time'                 => 'net.ipv4',
        'tcp_window_scaling'                 => 'net.ipv4',
        'tcp_sack'                           => 'net.ipv4',
        'accept_redirects'                   => 'net.ipv4.conf.all',
        'accept_source_route'                => 'net.ipv4.conf.all',
        'log_martians'                       => 'net.ipv4.conf.all',
        'rp_filter'                          => 'net.ipv4.conf.all',
    );

    while (my ($param,$name) = each %params) {
        push(@sysctl,"$name.$param = 1") if $Config{$param} =~ "on";
        push(@sysctl,"$name.$param = 0") if $Config{$param} =~ "off";
        push(@sysctl,"$name.$param = $Config{$param}") if $Config{$param} !~ "on" && $Config{$param} !~ "off";
    }

    if($Config{'overwrite_sysctl'} =~ "on") {
        open(SYSCTL,'>',$Config{'config_basedir'} . "/sysctl.conf");
        foreach my $param (@sysctl) {
            syswrite(SYSCTL,"$param\n");
        }
        close(SYSCTL);
    }
    my $file = $Config{'config_basedir'} . "/sysctl.conf";
    system("sysctl -p $file > /dev/null 2>&1");
    die $! if $?;
    return $?;
}

__END__
